__author__ = 'mnowotka'

from django.db import models
from django.db import connections
from django.db.models.query import QuerySet
from django.conf import settings

#-----------------------------------------------------------------------------------------------------------------------

class CompoundMolsMixin(object):

    def get_column(self, name):
        return filter(lambda x: x.name == name, self.model._meta.fields)[0].db_column or name

    def similar_to(self, structure, similarity_index, **kwargs):
        ctab_column = kwargs.get('ctab_column', self.get_column('ctab'))
        molregno_column = kwargs.get('molecule_column', self.get_column('molecule'))
        try:
            sim = int(similarity_index)
        except ValueError:
            raise ValueError('similarity_index must be integer from range (50,100)')
        if sim < 50 or sim > 100:
            raise ValueError('similarity_index must be integer from range (50,100)')

        connection = connections[self._db or 'default']
        if connection.vendor == 'oracle':
            return self.extra(select={'similarity': "TO_NUMBER (molsim (" + ctab_column + ",%s, 'normal'))"},
                select_params=(structure,),
                where=["molsim (" + ctab_column + ", %s, 'normal') BETWEEN %s AND '100'"],
                params=('smiles:' + structure, similarity_index))
        if connection.vendor == 'postgresql':
            fingerprints_table = kwargs.get('fingerprints_table', getattr(settings, 'FINGERPRINTS_TABLE', 'fps_rdkit'))
            fingerprints_pk = kwargs.get('fingerprints_pk', getattr(settings, 'FINGERPRINTS_PK', 'molregno'))
            similarity_type = kwargs.get('similarity_type', getattr(settings, 'SIMILARITY_TYPE', 'tanimoto_sml'))
            fingerprint_type = kwargs.get('fingerprint_type', getattr(settings, 'FINGERPRINT_TYPE', 'morganbv_fp'))
            fingerprint_column = kwargs.get('fingerprint_column', getattr(settings, 'FINGERPRINT_COLUMN', 'mfp2'))
            cursor = connection.cursor()
            cursor.execute("select 'c1ccccc1O'::mol;") # dirty hack but what can I do...
            cursor.execute('set rdkit.tanimoto_threshold=%s;', (sim / 100.0,))
            ret = self.extra(
                select={'similarity': similarity_type + "(" + fingerprint_type + "(%s), " + fingerprint_column + ")"},
                select_params=(structure,),
                where=[
                    fingerprints_table + "." + fingerprints_pk + " = " + self.model._meta.db_table + "." +
                    molregno_column + " and " + fingerprint_type + "(%s) %% " + fingerprint_column + " and " +
                    similarity_type + "(" + fingerprint_type + "(%s), " + fingerprint_column + ") between %s and 1.0"
                ],
                tables=[fingerprints_table],
                order_by=['-similarity'],
                params=[structure, structure, (sim / 100.0)])
            cursor.execute('set rdkit.tanimoto_threshold=0.5;')
            return ret
        else:
            raise NotImplementedError

    def with_substructure(self, structure):
        ctab_column = self.get_column('ctab')
        connection = connections[self._db or 'default']
        if connection.vendor == 'oracle':
            return self.extra(where=["(sss(" + ctab_column + ",%s,'ignore=all')=1)"], params=('smiles:' + structure,))
        if connection.vendor == 'postgresql':
            return self.extra(where=[ctab_column + "@>%s"], params=(structure,))
        else:
            raise NotImplementedError

    def flexmatch(self, structure):
        ctab_column = self.get_column('ctab')
        connection = connections[self._db or 'default']
        if connection.vendor == 'oracle':
            return self.extra(
                where=["(flexmatch(" + ctab_column + ",%s,'ignore=all')=1)"], params=('smiles:' + structure,))
        if connection.vendor == 'postgresql':
            try:
                from rdkit.rdBase import rdkitVersion
            except Exception:
                rdkitVersion = 0
            if rdkitVersion != '2015.03.1':
                return self.extra(where=[ctab_column + "@=%s"], params=(structure,))
            return self.extra(
                where=[ctab_column + "@>%s" + " AND " + ctab_column + "<@%s"], params=(structure, structure))
        else:
            raise NotImplementedError

#-----------------------------------------------------------------------------------------------------------------------

class CompoundMolsQuerySet(QuerySet,CompoundMolsMixin):
    pass

#-----------------------------------------------------------------------------------------------------------------------

class CompoundMolsManager(models.Manager, CompoundMolsMixin):
    use_for_related_fields = True
    def get_query_set(self):
        return CompoundMolsQuerySet(self.model, using=self._db)

#-----------------------------------------------------------------------------------------------------------------------


#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'mnowotka'

import sys

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

setup(
    name='chembl_core_db',
    version='0.7.0',
    author='Michal Nowotka',
    author_email='mnowotka@ebi.ac.uk',
    description='Core ChEMBL python library',
    url='https://www.ebi.ac.uk/chembl/',
    license='Apache Software License',
    packages=['chembl_core_db',
              'chembl_core_db.cache',
              'chembl_core_db.cache.backends',
              'chembl_core_db.db',
              'chembl_core_db.db.backends',
              'chembl_core_db.db.backends.oracleChEmbl',
              'chembl_core_db.db.models',
              'chembl_core_db.testing'],
    long_description=open('README.rst').read(),
    install_requires=['Django==1.5.5'],
    include_package_data=False,
    classifiers=['Development Status :: 4 - Beta',
                 'Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Apache Software License',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python :: 2.7',
                 'Topic :: Scientific/Engineering :: Chemistry'],
    zip_safe=False,
)



__author__ = 'mnowotka'

from django.db import models
from django.conf import settings
from django.db.models.base import ModelBase
import re
import sys
import inspect
from django.core.exceptions import ImproperlyConfigured
from django.db.models import get_model
import copy

#-----------------------------------------------------------------------------------------------------------------------

def convert(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()

#-----------------------------------------------------------------------------------------------------------------------

class ChemblCoreAbstractModel(models.Model):
    class Meta:
        abstract = True
        app_label = 'chembl_core_model'
        managed = settings.CORE_TABLES_MANAGED

#-----------------------------------------------------------------------------------------------------------------------

class ChemblAppAbstractModel(models.Model):
    class Meta:
        abstract = True
        managed = settings.APP_SPECIFIC_TABLES_MANAGED

#-----------------------------------------------------------------------------------------------------------------------

class ChemblModelMetaClass(ModelBase):
    def __new__(cls, name, bases, attrs):
        n = name
        if "Meta" in attrs:
            meta = attrs["Meta"]
            if hasattr(meta, "db_table"):
                n = meta.db_table
        klas = super(ChemblModelMetaClass, cls).__new__(cls, name, bases, attrs)
        if settings.EXPORT_MODE:
            klas._meta.db_table = unicode(convert(n))
        else:
            klas._meta.db_table = u'' + settings.CHEMBL_SCHEMA_NAME + '.' + convert(n) + ''

        return klas

#-----------------------------------------------------------------------------------------------------------------------

def remove_field(cls, f_name):
    if hasattr(cls, f_name):
        delattr(cls, f_name)

#-----------------------------------------------------------------------------------------------------------------------

def rebase(module, klas):
    if isinstance(klas, basestring):
        relClsName = klas.split('.')[-1]
    else:
        relClsName = klas.__name__

    lst =  inspect.getmembers(sys.modules[module], lambda x: inspect.isclass(x) and x.__name__ == relClsName)
    if len(lst):
        return lst[0][1]

    return module.split('.')[0] + '.' + relClsName

#-----------------------------------------------------------------------------------------------------------------------

class ModifiedModelMetaclass(ChemblModelMetaClass):

    def __new__(cls, name, bases, attrs):
        try:
            metaCls = attrs['Meta']
            meta = metaCls()
        except KeyError:
            raise ImproperlyConfigured("Helper class %s hasn't a Meta subclass!" % name)

        # Find model class for this helper
        try:
            model = getattr(meta, 'model')
        except AttributeError:
            return super(ModifiedModelMetaclass, cls).__new__(cls, name, bases, attrs)

        if isinstance(model, basestring):
            model_class = get_model(*model.split('.'))
        elif issubclass(model, models.Model):
            model_class = model
        else:
            raise ImproperlyConfigured("Model informed by Meta subclass of %s is improperly!" % name)
        remove_field(metaCls, 'model')

        module = attrs['__module__']
        excludes = getattr(meta, 'exclude', ())
        if excludes is None:
            excludes = ()
        remove_field(metaCls, 'exclude')
        attrs['Meta'] = metaCls

        fields = [f for f in model_class._meta.fields + model_class._meta.local_many_to_many if f.name not in excludes]
        for field in fields:
            f = copy.deepcopy(field)
            if hasattr(f, 'rel') and f.rel:
                if hasattr(f.rel, 'through'):
                    f.rel.through = rebase(module, f.rel.through)
                f.rel.to = rebase(module, f.rel.to)
            attrs[f.name] = f

        return super(ModifiedModelMetaclass, cls).__new__(cls, name, bases, attrs)

#-----------------------------------------------------------------------------------------------------------------------




from django.test.client import MULTIPART_CONTENT
from tastypie.test import TestApiClient as TastypieTestApiClient
from tastypie.test import ResourceTestCase as TastypieResourceTestCase

class TestApiClient(TastypieTestApiClient):

    def post(self, uri, format='json', data=None, authentication=None, content_type=MULTIPART_CONTENT, **kwargs):

        accept = self.get_content_type(format)
        kwargs['HTTP_ACCEPT'] = accept

        if content_type==MULTIPART_CONTENT:
            contentType = content_type
        else:
            contentType = self.get_content_type(content_type)
        kwargs['content_type'] = contentType

        if data is not None:
            if content_type==MULTIPART_CONTENT:
                kwargs['data'] = data
            else:
                kwargs['data'] = self.serializer.serialize(data, format=contentType)

        if authentication is not None:
            kwargs['HTTP_AUTHORIZATION'] = authentication

        return self.client.post(uri, **kwargs)

class ResourceTestCase(TastypieResourceTestCase):

    def assertHttpOK(self, resp, url = None):
        """
        Ensures the response is returning a HTTP 200.
        """
        return self.assertEqual(resp.status_code, 200, "response code is not 200 but " + str(resp.status_code) + " for url" + str(url))

    def assertValidJSON(self, data, url = None):
        """
        Given the provided ``data`` as a string, ensures that it is valid JSON &
        can be loaded properly.
        """
        # Just try the load. If it throws an exception, the test case will fail.
        try:
            self.serializer.from_json(data)
        except:
            raise Exception("Can't serialise data for url" + url)

    def assertValidJSONResponse(self, resp, url = None):
        """
        Given a ``HttpResponse`` coming back from using the ``client``, assert that
        you get back:

        * An HTTP 200
        * The correct content-type (``application/json``)
        * The content is valid JSON
        """
        self.assertHttpOK(resp, url)
        self.assertTrue(resp['Content-Type'].startswith('application/json'), "content-Type is not application/json but " + resp['Content-Type'] + " for url " + str(url))
        self.assertValidJSON(resp.content, url)